@@ -1,14 +1,11 @@
 /*
  * Device scanning mini-daemon for CUPS.
  *
- * Copyright 2007-2014 by Apple Inc.
- * Copyright 1997-2006 by Easy Software Products.
+ * Copyright © 2007-2018 by Apple Inc.
+ * Copyright © 1997-2006 by Easy Software Products.
  *
- * These coded instructions, statements, and computer programs are the
- * property of Apple Inc. and are protected by Federal copyright
- * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- * which should have been included with this file.  If this file is
- * missing or damaged, see the license at "http://www.cups.org/".
+ * Licensed under Apache License v2.0.  See the file "LICENSE" for more
+ * information.
  */
 
 /*
@@ -169,10 +166,10 @@ main(int  argc,				/* I - Number of command-line args */
     return (1);
   }
 
-  normal_user = (uid_t)atoi(argv[4]);
-  if (normal_user <= 0)
+  normal_user = (uid_t)strtoul(argv[4], NULL, 10);
+  if (normal_user == 0)
   {
-    fprintf(stderr, "ERROR: [cups-deviced] Bad user %d!\n", normal_user);
+    fprintf(stderr, "ERROR: [cups-deviced] Bad user %u!\n", (unsigned)normal_user);
 
     return (1);
   }
@@ -268,7 +265,7 @@ main(int  argc,				/* I - Number of command-line args */
     * all others run as the unprivileged user...
     */
 
-    start_backend(dent->filename, !(dent->fileinfo.st_mode & (S_IWGRP | S_IRWXO)));
+    start_backend(dent->filename, !(dent->fileinfo.st_mode & (S_IWGRP | S_IWOTH | S_IXOTH)));
   }
 
   cupsDirClose(dir);
@@ -312,7 +309,7 @@ main(int  argc,				/* I - Number of command-line args */
 	      break;
 	    }
 	  }
-	  while (bpipe->ptr && memchr(bpipe->ptr, '\n', (size_t)(bpipe->end - bpipe->ptr)));
+	  while (_cupsFilePeekAhead(bpipe, '\n'));
         }
     }
 
